'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib_1 = require('tslib');
var React = require('react');
var isEqual = _interopDefault(require('react-fast-compare'));
var deepmerge = _interopDefault(require('deepmerge'));
var clone = _interopDefault(require('lodash/clone'));
var toPath = _interopDefault(require('lodash/toPath'));
var invariant = _interopDefault(require('tiny-warning'));
var hoistNonReactStatics = _interopDefault(require('hoist-non-react-statics'));
var cloneDeep = _interopDefault(require('lodash/cloneDeep'));

var isFunction = function (obj) {
    return typeof obj === 'function';
};
var isObject = function (obj) {
    return obj !== null && typeof obj === 'object';
};
var isInteger = function (obj) {
    return String(Math.floor(Number(obj))) === obj;
};
var isString = function (obj) {
    return Object.prototype.toString.call(obj) === '[object String]';
};
var isNaN$1 = function (obj) { return obj !== obj; };
var isEmptyChildren = function (children) {
    return React.Children.count(children) === 0;
};
var isPromise = function (value) {
    return isObject(value) && isFunction(value.then);
};
var isInputEvent = function (value) {
    return value && isObject(value) && isObject(value.target);
};
function getActiveElement(doc) {
    doc = doc || (typeof document !== 'undefined' ? document : undefined);
    if (typeof doc === 'undefined') {
        return null;
    }
    try {
        return doc.activeElement || doc.body;
    }
    catch (e) {
        return doc.body;
    }
}
function makeCancelable(promise) {
    var hasCanceled = false;
    var wrappedPromise = new Promise(function (resolve, reject) {
        promise.then(function (val) { return (hasCanceled ? reject({ isCanceled: true }) : resolve(val)); }, function (error) { return (hasCanceled ? reject({ isCanceled: true }) : reject(error)); });
    });
    return [
        wrappedPromise,
        function cancel() {
            hasCanceled = true;
        },
    ];
}
function getIn(obj, key, def, p) {
    if (p === void 0) { p = 0; }
    var path = toPath(key);
    while (obj && p < path.length) {
        obj = obj[path[p++]];
    }
    return obj === undefined ? def : obj;
}
function setIn(obj, path, value) {
    var res = clone(obj);
    var resVal = res;
    var i = 0;
    var pathArray = toPath(path);
    for (; i < pathArray.length - 1; i++) {
        var currentPath = pathArray[i];
        var currentObj = getIn(obj, pathArray.slice(0, i + 1));
        if (currentObj) {
            resVal = resVal[currentPath] = clone(currentObj);
        }
        else {
            var nextPath = pathArray[i + 1];
            resVal = resVal[currentPath] =
                isInteger(nextPath) && Number(nextPath) >= 0 ? [] : {};
        }
    }
    if ((i === 0 ? obj : resVal)[pathArray[i]] === value) {
        return obj;
    }
    if (value === undefined) {
        delete resVal[pathArray[i]];
    }
    else {
        resVal[pathArray[i]] = value;
    }
    if (i === 0 && value === undefined) {
        delete res[pathArray[i]];
    }
    return res;
}
function setNestedObjectValues(object, value, visited, response) {
    if (visited === void 0) { visited = new WeakMap(); }
    if (response === void 0) { response = {}; }
    for (var _i = 0, _a = Object.keys(object); _i < _a.length; _i++) {
        var k = _a[_i];
        var val = object[k];
        if (isObject(val)) {
            if (!visited.get(val)) {
                visited.set(val, true);
                response[k] = Array.isArray(val) ? [] : {};
                setNestedObjectValues(val, value, visited, response[k]);
            }
        }
        else {
            response[k] = value;
        }
    }
    return response;
}

var PrivateFormikContext = React.createContext({});
var FormikProvider = PrivateFormikContext.Provider;
var FormikConsumer = PrivateFormikContext.Consumer;
function useFormikContext() {
    return React.useContext(PrivateFormikContext);
}

function formikReducer(state, msg) {
    switch (msg.type) {
        case 'SET_VALUES':
            return tslib_1.__assign({}, state, { values: msg.payload });
        case 'SET_TOUCHED':
            return tslib_1.__assign({}, state, { touched: msg.payload });
        case 'SET_ERRORS':
            return tslib_1.__assign({}, state, { errors: msg.payload });
        case 'SET_STATUS':
            return tslib_1.__assign({}, state, { status: msg.payload });
        case 'SET_ISSUBMITTING':
            return tslib_1.__assign({}, state, { isSubmitting: msg.payload });
        case 'SET_ISVALIDATING':
            return tslib_1.__assign({}, state, { isValidating: msg.payload });
        case 'SET_FIELD_VALUE':
            return tslib_1.__assign({}, state, { values: setIn(state.values, msg.payload.field, msg.payload.value) });
        case 'SET_FIELD_TOUCHED':
            return tslib_1.__assign({}, state, { touched: setIn(state.touched, msg.payload.field, msg.payload.value) });
        case 'SET_FIELD_ERROR':
            return tslib_1.__assign({}, state, { errors: setIn(state.errors, msg.payload.field, msg.payload.value) });
        case 'RESET_FORM':
        case 'SET_FORMIK_STATE':
            return tslib_1.__assign({}, state, msg.payload);
        case 'SUBMIT_ATTEMPT':
            return tslib_1.__assign({}, state, { touched: setNestedObjectValues(state.values, true), isSubmitting: true, submitCount: state.submitCount + 1 });
        case 'SUBMIT_FAILURE':
            return tslib_1.__assign({}, state, { isSubmitting: false });
        case 'SUBMIT_SUCCESS':
            return tslib_1.__assign({}, state, { isSubmitting: false });
        default:
            return state;
    }
}
function usePrevious(val) {
    var ref = React.useRef(val);
    React.useEffect(function () {
        ref.current = val;
    }, [val]);
    return ref.current;
}
function useFormik(_a) {
    var _b = _a.validateOnChange, validateOnChange = _b === void 0 ? true : _b, _c = _a.validateOnBlur, validateOnBlur = _c === void 0 ? true : _c, isInitialValid = _a.isInitialValid, _d = _a.enableReinitialize, enableReinitialize = _d === void 0 ? false : _d, onSubmit = _a.onSubmit, rest = tslib_1.__rest(_a, ["validateOnChange", "validateOnBlur", "isInitialValid", "enableReinitialize", "onSubmit"]);
    var props = tslib_1.__assign({ validateOnChange: validateOnChange, validateOnBlur: validateOnBlur, onSubmit: onSubmit }, rest);
    var initialValues = React.useRef(props.initialValues);
    var initialErrors = React.useRef(props.initialErrors || {});
    var initialTouched = React.useRef(props.initialTouched || {});
    var initialStatus = React.useRef(props.initialStatus);
    var isMounted = React.useRef(false);
    var fields = React.useRef({});
    invariant(typeof isInitialValid === 'undefined', 'isInitialValid has been deprecated and will be removed in future versions of Formik. Please use initialErrors instead.');
    React.useEffect(function () {
        isMounted.current = true;
        return function () {
            isMounted.current = false;
        };
    }, []);
    var _e = React.useReducer(formikReducer, {
        values: props.initialValues,
        errors: props.initialErrors || {},
        touched: props.initialTouched || {},
        status: props.initialStatus,
        isSubmitting: false,
        isValidating: false,
        submitCount: 0,
    }), state = _e[0], dispatch = _e[1];
    var prevState = usePrevious(state);
    var runValidateHandler = React.useCallback(function (values, field) {
        return new Promise(function (resolve) {
            var maybePromisedErrors = props.validate(values, field);
            if (maybePromisedErrors === undefined) {
                resolve({});
            }
            else if (isPromise(maybePromisedErrors)) {
                maybePromisedErrors.then(function () {
                    resolve({});
                }, function (errors) {
                    resolve(errors);
                });
            }
            else {
                resolve(maybePromisedErrors);
            }
        });
    }, [props.validate]);
    var runValidationSchema = React.useCallback(function (values, field) {
        return new Promise(function (resolve) {
            var validationSchema = props.validationSchema;
            var schema = isFunction(validationSchema)
                ? validationSchema(field)
                : validationSchema;
            var promise = field && schema.validateAt
                ? schema.validateAt(field, values)
                : validateYupSchema(values, schema);
            promise.then(function () {
                resolve({});
            }, function (err) {
                resolve(yupToFormErrors(err));
            });
        });
    }, [props.validationSchema]);
    var runSingleFieldLevelValidation = React.useCallback(function (field, value) {
        return new Promise(function (resolve) {
            return resolve(fields.current[field].validate(value));
        }).then(function (x) { return x; }, function (e) { return e; });
    }, [fields]);
    var runFieldLevelValidations = React.useCallback(function (values) {
        if (fields.current === null) {
            return Promise.resolve({});
        }
        var fieldKeysWithValidation = Object.keys(fields.current).filter(function (f) {
            return fields.current !== null &&
                fields.current[f] &&
                fields.current[f].validate &&
                isFunction(fields.current[f].validate);
        });
        var fieldValidations = fieldKeysWithValidation.length > 0
            ? fieldKeysWithValidation.map(function (f) {
                return runSingleFieldLevelValidation(f, getIn(values, f));
            })
            : [Promise.resolve('DO_NOT_DELETE_YOU_WILL_BE_FIRED')];
        return Promise.all(fieldValidations).then(function (fieldErrorsList) {
            return fieldErrorsList.reduce(function (prev, curr, index) {
                if (curr === 'DO_NOT_DELETE_YOU_WILL_BE_FIRED') {
                    return prev;
                }
                if (curr) {
                    prev = setIn(prev, fieldKeysWithValidation[index], curr);
                }
                return prev;
            }, {});
        });
    }, [runSingleFieldLevelValidation, fields]);
    var validateForm = React.useCallback(function (values) {
        if (values === void 0) { values = state.values; }
        if (props.validationSchema ||
            props.validate ||
            (fields.current &&
                Object.keys(fields.current).filter(function (key) { return !!fields.current[key].validate; }).length > 0)) {
            dispatch({ type: 'SET_ISVALIDATING', payload: true });
            return Promise.all([
                runFieldLevelValidations(values),
                props.validationSchema ? runValidationSchema(values) : {},
                props.validate ? runValidateHandler(values) : {},
            ]).then(function (_a) {
                var fieldErrors = _a[0], schemaErrors = _a[1], validateErrors = _a[2];
                var combinedErrors = deepmerge.all([fieldErrors, schemaErrors, validateErrors], { arrayMerge: arrayMerge });
                if (!isEqual(state.errors, combinedErrors)) {
                    dispatch({ type: 'SET_ERRORS', payload: combinedErrors });
                }
                dispatch({ type: 'SET_ISVALIDATING', payload: false });
                return combinedErrors;
            });
        }
        else {
            return Promise.resolve({});
        }
    }, [
        props.validate,
        props.validationSchema,
        runFieldLevelValidations,
        runValidateHandler,
        runValidationSchema,
        state.errors,
        state.values,
        fields,
    ]);
    React.useEffect(function () {
        if (prevState.values !== state.values &&
            !!validateOnChange &&
            !state.isSubmitting &&
            isMounted.current != null) {
            var _a = makeCancelable(validateForm()), validate = _a[0], cancel = _a[1];
            validate.then(function (x) { return x; }).catch(function (x) { return x; });
            return cancel;
        }
        return;
    }, [
        prevState.values,
        state.isSubmitting,
        state.values,
        validateForm,
        validateOnChange,
        isMounted,
    ]);
    React.useEffect(function () {
        if (prevState.touched !== state.touched &&
            !!validateOnBlur &&
            !state.isSubmitting &&
            isMounted.current != null) {
            var _a = makeCancelable(validateForm()), validate = _a[0], cancel = _a[1];
            validate.then(function (x) { return x; }).catch(function (x) { return x; });
            return cancel;
        }
        return;
    }, [
        prevState.touched,
        state.isSubmitting,
        state.touched,
        validateForm,
        validateOnBlur,
        isMounted,
    ]);
    var resetForm = React.useCallback(function (nextState) {
        var values = nextState && nextState.values
            ? nextState.values
            : initialValues.current
                ? initialValues.current
                : props.initialValues;
        var errors = nextState && nextState.errors
            ? nextState.values
            : initialErrors.current
                ? initialErrors.current
                : props.initialErrors || {};
        var touched = nextState && nextState.touched
            ? nextState.values
            : initialTouched.current
                ? initialTouched.current
                : props.initialTouched || {};
        var status = nextState && nextState.status
            ? nextState.status
            : initialStatus.current
                ? initialStatus.current
                : props.initialStatus;
        initialValues.current = values;
        initialErrors.current = errors;
        initialTouched.current = touched;
        initialStatus.current = status;
        dispatch({
            type: 'RESET_FORM',
            payload: {
                isSubmitting: !!nextState && !!nextState.isSubmitting,
                errors: errors,
                touched: touched,
                status: status,
                values: values,
                isValidating: !!nextState && !!nextState.isValidating,
                submitCount: !!nextState &&
                    !!nextState.submitCount &&
                    typeof nextState.submitCount === 'number'
                    ? nextState.submitCount
                    : 0,
            },
        });
    }, [
        props.initialErrors,
        props.initialStatus,
        props.initialTouched,
        props.initialValues,
    ]);
    React.useEffect(function () {
        if (enableReinitialize &&
            isMounted.current &&
            !isEqual(initialValues.current, props.initialValues)) {
            resetForm();
        }
    }, [enableReinitialize, props.initialValues, resetForm]);
    var validateField = React.useCallback(function (name) {
        if (fields.current !== null &&
            fields.current[name] &&
            fields.current[name].validate &&
            isFunction(fields.current[name].validate)) {
            var value = getIn(state.values, name);
            var maybePromise = fields.current[name].validate(value);
            if (isPromise(maybePromise)) {
                dispatch({ type: 'SET_ISVALIDATING', payload: true });
                return maybePromise
                    .then(function (x) { return x; }, function (e) { return e; })
                    .then(function (error) {
                    dispatch({
                        type: 'SET_FIELD_ERROR',
                        payload: { field: name, value: error },
                    });
                    dispatch({ type: 'SET_ISVALIDATING', payload: false });
                });
            }
            else {
                dispatch({
                    type: 'SET_FIELD_ERROR',
                    payload: {
                        field: name,
                        value: maybePromise,
                    },
                });
                return Promise.resolve(maybePromise);
            }
        }
        else {
            return Promise.resolve();
        }
    }, [state.values, fields]);
    var registerField = React.useCallback(function (name, _a) {
        var validate = _a.validate;
        if (fields.current !== null) {
            fields.current[name] = {
                validate: validate,
            };
        }
    }, [fields]);
    var unregisterField = React.useCallback(function (name) {
        if (fields.current !== null) {
            delete fields.current[name];
        }
    }, [fields]);
    var handleBlur = React.useCallback(function (eventOrString) {
        if (isString(eventOrString)) {
            return function (event) { return executeBlur(event, eventOrString); };
        }
        else {
            executeBlur(eventOrString);
        }
        function executeBlur(e, path) {
            if (e.persist) {
                e.persist();
            }
            var _a = e.target, name = _a.name, id = _a.id, outerHTML = _a.outerHTML;
            var field = path ? path : name ? name : id;
            if (!field && "development" !== 'production') {
                warnAboutMissingIdentifier({
                    htmlContent: outerHTML,
                    documentationAnchorLink: 'handleblur-e-any--void',
                    handlerName: 'handleBlur',
                });
            }
            dispatch({
                type: 'SET_FIELD_TOUCHED',
                payload: { field: field, value: true },
            });
        }
    }, []);
    var handleChange = React.useCallback(function (eventOrPath) {
        if (isString(eventOrPath)) {
            return function (event) { return executeChange(event, eventOrPath); };
        }
        else {
            executeChange(eventOrPath);
        }
        function executeChange(eventOrTextValue, maybePath) {
            var field = maybePath;
            var val = eventOrTextValue;
            var parsed;
            if (!isString(eventOrTextValue)) {
                if (eventOrTextValue.persist) {
                    eventOrTextValue.persist();
                }
                var _a = eventOrTextValue.target, type = _a.type, name_1 = _a.name, id = _a.id, value = _a.value, checked = _a.checked, outerHTML = _a.outerHTML;
                field = maybePath ? maybePath : name_1 ? name_1 : id;
                if (!field && "development" !== 'production') {
                    warnAboutMissingIdentifier({
                        htmlContent: outerHTML,
                        documentationAnchorLink: 'handlechange-e-reactchangeeventany--void',
                        handlerName: 'handleChange',
                    });
                }
                val = /number|range/.test(type)
                    ? ((parsed = parseFloat(value)), isNaN(parsed) ? '' : parsed)
                    : /checkbox/.test(type)
                        ? checked
                        : value;
            }
            if (field) {
                dispatch({ type: 'SET_FIELD_VALUE', payload: { field: field, value: val } });
            }
        }
    }, []);
    function handleSubmit(e) {
        if (e && e.preventDefault) {
            e.preventDefault();
        }
        if (typeof document !== 'undefined') {
            var activeElement = getActiveElement();
            if (activeElement !== null &&
                activeElement instanceof HTMLButtonElement) {
                invariant(activeElement.attributes &&
                    activeElement.attributes.getNamedItem('type'), 'You submitted a Formik form using a button with an unspecified `type` attribute.  Most browsers default button elements to `type="submit"`. If this is not a submit button, please add `type="button"`.');
            }
        }
        submitForm();
    }
    var setTouched = React.useCallback(function (touched) {
        dispatch({ type: 'SET_TOUCHED', payload: touched });
    }, []);
    var setErrors = React.useCallback(function (errors) {
        dispatch({ type: 'SET_ERRORS', payload: errors });
    }, []);
    var setValues = React.useCallback(function (values) {
        dispatch({ type: 'SET_VALUES', payload: values });
    }, []);
    var setFieldError = React.useCallback(function (field, value) {
        dispatch({
            type: 'SET_FIELD_ERROR',
            payload: { field: field, value: value },
        });
    }, []);
    var setFieldValue = React.useCallback(function (field, value) {
        dispatch({
            type: 'SET_FIELD_VALUE',
            payload: {
                field: field,
                value: value,
            },
        });
    }, []);
    var setFieldTouched = React.useCallback(function (field, touched) {
        if (touched === void 0) { touched = true; }
        dispatch({
            type: 'SET_FIELD_TOUCHED',
            payload: {
                field: field,
                value: touched,
            },
        });
    }, []);
    function setFormikState(stateOrCb) {
        if (isFunction(stateOrCb)) {
            dispatch({ type: 'SET_FORMIK_STATE', payload: stateOrCb(state) });
        }
        else {
            dispatch({ type: 'SET_FORMIK_STATE', payload: stateOrCb });
        }
    }
    var setStatus = React.useCallback(function (status) {
        dispatch({ type: 'SET_STATUS', payload: status });
    }, []);
    var setSubmitting = React.useCallback(function (isSubmitting) {
        dispatch({ type: 'SET_ISSUBMITTING', payload: isSubmitting });
    }, []);
    var imperativeMethods = {
        resetForm: resetForm,
        validateForm: validateForm,
        validateField: validateField,
        setErrors: setErrors,
        setFieldError: setFieldError,
        setFieldTouched: setFieldTouched,
        setFieldValue: setFieldValue,
        setStatus: setStatus,
        setSubmitting: setSubmitting,
        setTouched: setTouched,
        setValues: setValues,
        setFormikState: setFormikState,
    };
    var executeSubmit = React.useCallback(function () {
        return onSubmit(state.values, imperativeMethods);
    }, [imperativeMethods, onSubmit, state.values]);
    var submitForm = React.useCallback(function () {
        dispatch({ type: 'SUBMIT_ATTEMPT' });
        return validateForm().then(function (combinedErrors) {
            var isActuallyValid = Object.keys(combinedErrors).length === 0;
            if (isActuallyValid) {
                Promise.resolve(executeSubmit())
                    .then(function () {
                    if (isMounted.current) {
                        dispatch({ type: 'SUBMIT_SUCCESS' });
                    }
                })
                    .catch(function (_errors) {
                    if (isMounted.current) {
                        dispatch({ type: 'SUBMIT_FAILURE' });
                    }
                });
            }
            else if (isMounted.current) {
                dispatch({ type: 'SUBMIT_FAILURE' });
            }
        });
    }, [executeSubmit, validateForm]);
    var handleReset = React.useCallback(function () {
        if (props.onReset) {
            var maybePromisedOnReset = props.onReset(state.values, imperativeMethods);
            if (isPromise(maybePromisedOnReset)) {
                maybePromisedOnReset.then(resetForm);
            }
            else {
                resetForm();
            }
        }
        else {
            resetForm();
        }
    }, [imperativeMethods, props.onReset, resetForm, state.values]);
    var getFieldMeta = React.useCallback(function (name) {
        return {
            value: getIn(state.values, name),
            error: getIn(state.errors, name),
            touched: !!getIn(state.touched, name),
            initialValue: getIn(initialValues.current, name),
            initialTouched: !!getIn(initialTouched.current, name),
            initialError: getIn(initialErrors.current, name),
        };
    }, [state.errors, state.touched, state.values]);
    var getFieldProps = React.useCallback(function (name, type) {
        var field = {
            name: name,
            value: type && (type === 'radio' || type === 'checkbox')
                ? undefined
                : getIn(state.values, name),
            onChange: handleChange,
            onBlur: handleBlur,
        };
        return [field, getFieldMeta(name)];
    }, [getFieldMeta, handleBlur, handleChange, state.values]);
    var dirty = React.useMemo(function () { return !isEqual(initialValues.current, state.values); }, [state.values]);
    var isValid = React.useMemo(function () {
        return typeof isInitialValid !== 'undefined'
            ? dirty
                ? state.errors && Object.keys(state.errors).length === 0
                : isInitialValid !== false && isFunction(isInitialValid)
                    ? isInitialValid(props)
                    : isInitialValid
            : state.errors && Object.keys(state.errors).length === 0;
    }, [isInitialValid, dirty, state.errors, props]);
    var ctx = tslib_1.__assign({}, state, { initialValues: initialValues.current, initialErrors: initialErrors.current, initialTouched: initialTouched.current, initialStatus: initialStatus.current, handleBlur: handleBlur,
        handleChange: handleChange,
        handleReset: handleReset,
        handleSubmit: handleSubmit,
        resetForm: resetForm,
        setErrors: setErrors,
        setFormikState: setFormikState,
        setFieldTouched: setFieldTouched,
        setFieldValue: setFieldValue,
        setFieldError: setFieldError,
        setStatus: setStatus,
        setSubmitting: setSubmitting,
        setTouched: setTouched,
        setValues: setValues,
        submitForm: submitForm,
        validateForm: validateForm,
        validateField: validateField,
        isValid: isValid,
        dirty: dirty,
        unregisterField: unregisterField,
        registerField: registerField,
        getFieldProps: getFieldProps,
        validateOnBlur: validateOnBlur,
        validateOnChange: validateOnChange });
    return ctx;
}
function Formik(props) {
    var formikbag = useFormik(props);
    var component = props.component, children = props.children, render = props.render;
    return (React.createElement(FormikProvider, { value: formikbag }, component
        ? React.createElement(component, formikbag)
        : render
            ? render(formikbag)
            : children
                ? isFunction(children)
                    ? children(formikbag)
                    : !isEmptyChildren(children)
                        ? React.Children.only(children)
                        : null
                : null));
}
function warnAboutMissingIdentifier(_a) {
    var htmlContent = _a.htmlContent, documentationAnchorLink = _a.documentationAnchorLink, handlerName = _a.handlerName;
    console.warn("Warning: Formik called `" + handlerName + "`, but you forgot to pass an `id` or `name` attribute to your input:\n    " + htmlContent + "\n    Formik cannot determine which value to update. For more info see https://github.com/jaredpalmer/formik#" + documentationAnchorLink + "\n  ");
}
function yupToFormErrors(yupError) {
    var errors = {};
    if (yupError.inner.length === 0) {
        return setIn(errors, yupError.path, yupError.message);
    }
    for (var _i = 0, _a = yupError.inner; _i < _a.length; _i++) {
        var err = _a[_i];
        if (!errors[err.path]) {
            errors = setIn(errors, err.path, err.message);
        }
    }
    return errors;
}
function validateYupSchema(values, schema, sync, context) {
    if (sync === void 0) { sync = false; }
    if (context === void 0) { context = {}; }
    var validateData = {};
    for (var k in values) {
        if (values.hasOwnProperty(k)) {
            var key = String(k);
            validateData[key] = values[key] !== '' ? values[key] : undefined;
        }
    }
    return schema[sync ? 'validateSync' : 'validate'](validateData, {
        abortEarly: false,
        context: context,
    });
}
function arrayMerge(target, source, options) {
    var destination = target.slice();
    source.forEach(function (e, i) {
        if (typeof destination[i] === 'undefined') {
            var cloneRequested = options.clone !== false;
            var shouldClone = cloneRequested && options.isMergeableObject(e);
            destination[i] = shouldClone
                ? deepmerge(Array.isArray(e) ? [] : {}, e, options)
                : e;
        }
        else if (options.isMergeableObject(e)) {
            destination[i] = deepmerge(target[i], e, options);
        }
        else if (target.indexOf(e) === -1) {
            destination.push(e);
        }
    });
    return destination;
}

function useField(name, type) {
    var formik = useFormikContext();
    invariant(formik, 'useField() / <Field /> must be used underneath a <Formik> component or withFormik() higher order component');
    return formik.getFieldProps(name, type);
}
function Field(_a) {
    var validate = _a.validate, name = _a.name, render = _a.render, children = _a.children, _b = _a.as, is = _b === void 0 ? 'input' : _b, component = _a.component, props = tslib_1.__rest(_a, ["validate", "name", "render", "children", "as", "component"]);
    var _c = useFormikContext(), _validate = _c.validate, _validationSchema = _c.validationSchema, formik = tslib_1.__rest(_c, ["validate", "validationSchema"]);
    React.useEffect(function () {
        invariant(!render, "<Field render> has been deprecated and will be removed in future versions of Formik. Please use a child callback function instead. To get rid of this warning, replace <Field name=\"" + name + "\" render={({field, form}) => ...} /> with <Field name=\"" + name + "\">{({field, form, meta}) => ...}</Field>");
        invariant(!component, '<Field component> has been deprecated and will be removed in future versions of Formik. Use <Field as> instead. Note that with the `as` prop, all props are passed directly through and not grouped in `field` object key.');
        invariant(!(is && children && isFunction(children)), 'You should not use <Field as> and <Field children> as a function in the same <Field> component; <Field as> will be ignored.');
        invariant(!(component && children && isFunction(children)), 'You should not use <Field component> and <Field children> as a function in the same <Field> component; <Field component> will be ignored.');
        invariant(!(render && children && !isEmptyChildren(children)), 'You should not use <Field render> and <Field children> in the same <Field> component; <Field children> will be ignored');
    }, [children, component, is, name, render]);
    React.useEffect(function () {
        formik.registerField(name, {
            validate: validate,
        });
        return function () {
            formik.unregisterField(name);
        };
    }, [formik, name, validate]);
    var _d = formik.getFieldProps(name, props.type), field = _d[0], meta = _d[1];
    var legacyBag = { field: field, form: formik };
    if (render) {
        return render(legacyBag);
    }
    if (isFunction(children)) {
        return children(tslib_1.__assign({}, legacyBag, { meta: meta }));
    }
    if (component) {
        if (typeof component === 'string') {
            var innerRef = props.innerRef, rest = tslib_1.__rest(props, ["innerRef"]);
            return React.createElement(component, tslib_1.__assign({ ref: innerRef }, field, rest), children);
        }
        return React.createElement(component, tslib_1.__assign({ field: field, form: formik }, props), children);
    }
    if (typeof is === 'string') {
        var innerRef = props.innerRef, rest = tslib_1.__rest(props, ["innerRef"]);
        return React.createElement(is, tslib_1.__assign({ ref: innerRef }, field, rest), children);
    }
    return React.createElement(is, tslib_1.__assign({}, field, props), children);
}
var FastField = Field;

function Form(props) {
    var _a = useFormikContext(), handleReset = _a.handleReset, handleSubmit = _a.handleSubmit;
    return React.createElement("form", tslib_1.__assign({ onSubmit: handleSubmit, onReset: handleReset }, props));
}
Form.displayName = 'Form';

function withFormik(_a) {
    var _b = _a.mapPropsToValues, mapPropsToValues = _b === void 0 ? function (vanillaProps) {
        var val = {};
        for (var k in vanillaProps) {
            if (vanillaProps.hasOwnProperty(k) &&
                typeof vanillaProps[k] !== 'function') {
                val[k] = vanillaProps[k];
            }
        }
        return val;
    } : _b, config = tslib_1.__rest(_a, ["mapPropsToValues"]);
    return function createFormik(Component) {
        var componentDisplayName = Component.displayName ||
            Component.name ||
            (Component.constructor && Component.constructor.name) ||
            'Component';
        var C = (function (_super) {
            tslib_1.__extends(C, _super);
            function C() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.validate = function (values) {
                    return config.validate(values, _this.props);
                };
                _this.validationSchema = function () {
                    return isFunction(config.validationSchema)
                        ? config.validationSchema(_this.props)
                        : config.validationSchema;
                };
                _this.handleSubmit = function (values, actions) {
                    return config.handleSubmit(values, tslib_1.__assign({}, actions, { props: _this.props }));
                };
                _this.renderFormComponent = function (formikProps) {
                    return React.createElement(Component, tslib_1.__assign({}, _this.props, formikProps));
                };
                return _this;
            }
            C.prototype.render = function () {
                var _a = this.props, children = _a.children, props = tslib_1.__rest(_a, ["children"]);
                return (React.createElement(Formik, tslib_1.__assign({}, props, config, { validate: config.validate && this.validate, validationSchema: config.validationSchema && this.validationSchema, initialValues: mapPropsToValues(this.props), initialStatus: config.mapPropsToStatus && config.mapPropsToStatus(this.props), initialErrors: config.mapPropsToErrors && config.mapPropsToErrors(this.props), initialTouched: config.mapPropsToTouched && config.mapPropsToTouched(this.props), onSubmit: this.handleSubmit, render: this.renderFormComponent })));
            };
            C.displayName = "WithFormik(" + componentDisplayName + ")";
            return C;
        }(React.Component));
        return hoistNonReactStatics(C, Component);
    };
}

function connect(Comp) {
    var C = function (props) { return (React.createElement(FormikConsumer, null, function (formik) { return React.createElement(Comp, tslib_1.__assign({}, props, { formik: formik })); })); };
    var componentDisplayName = Comp.displayName ||
        Comp.name ||
        (Comp.constructor && Comp.constructor.name) ||
        'Component';
    C.WrappedComponent = Comp;
    C.displayName = "FormikConnect(" + componentDisplayName + ")";
    return hoistNonReactStatics(C, Comp);
}

var move = function (array, from, to) {
    var copy = (array || []).slice();
    var value = copy[from];
    copy.splice(from, 1);
    copy.splice(to, 0, value);
    return copy;
};
var swap = function (array, indexA, indexB) {
    var copy = (array || []).slice();
    var a = copy[indexA];
    copy[indexA] = copy[indexB];
    copy[indexB] = a;
    return copy;
};
var insert = function (array, index, value) {
    var copy = (array || []).slice();
    copy.splice(index, 0, value);
    return copy;
};
var replace = function (array, index, value) {
    var copy = (array || []).slice();
    copy[index] = value;
    return copy;
};
var FieldArrayInner = (function (_super) {
    tslib_1.__extends(FieldArrayInner, _super);
    function FieldArrayInner(props) {
        var _this = _super.call(this, props) || this;
        _this.updateArrayField = function (fn, alterTouched, alterErrors) {
            var _a = _this.props, name = _a.name, validateOnChange = _a.validateOnChange, _b = _a.formik, setFormikState = _b.setFormikState, validateForm = _b.validateForm;
            setFormikState(function (prevState) {
                var updateErrors = typeof alterErrors === 'function' ? alterErrors : fn;
                var updateTouched = typeof alterTouched === 'function' ? alterTouched : fn;
                return tslib_1.__assign({}, prevState, { values: setIn(prevState.values, name, fn(getIn(prevState.values, name))), errors: alterErrors
                        ? setIn(prevState.errors, name, updateErrors(getIn(prevState.errors, name)))
                        : prevState.errors, touched: alterTouched
                        ? setIn(prevState.touched, name, updateTouched(getIn(prevState.touched, name)))
                        : prevState.touched });
            }, function () {
                if (validateOnChange) {
                    validateForm();
                }
            });
        };
        _this.push = function (value) {
            return _this.updateArrayField(function (array) { return (array || []).concat([cloneDeep(value)]); }, false, false);
        };
        _this.handlePush = function (value) { return function () { return _this.push(value); }; };
        _this.swap = function (indexA, indexB) {
            return _this.updateArrayField(function (array) { return swap(array, indexA, indexB); }, true, true);
        };
        _this.handleSwap = function (indexA, indexB) { return function () {
            return _this.swap(indexA, indexB);
        }; };
        _this.move = function (from, to) {
            return _this.updateArrayField(function (array) { return move(array, from, to); }, true, true);
        };
        _this.handleMove = function (from, to) { return function () { return _this.move(from, to); }; };
        _this.insert = function (index, value) {
            return _this.updateArrayField(function (array) { return insert(array, index, value); }, function (array) { return insert(array, index, null); }, function (array) { return insert(array, index, null); });
        };
        _this.handleInsert = function (index, value) { return function () { return _this.insert(index, value); }; };
        _this.replace = function (index, value) {
            return _this.updateArrayField(function (array) { return replace(array, index, value); }, false, false);
        };
        _this.handleReplace = function (index, value) { return function () {
            return _this.replace(index, value);
        }; };
        _this.unshift = function (value) {
            var length = -1;
            _this.updateArrayField(function (array) {
                var arr = array ? [value].concat(array) : [value];
                if (length < 0) {
                    length = arr.length;
                }
                return arr;
            }, function (array) {
                var arr = array ? [null].concat(array) : [null];
                if (length < 0) {
                    length = arr.length;
                }
                return arr;
            }, function (array) {
                var arr = array ? [null].concat(array) : [null];
                if (length < 0) {
                    length = arr.length;
                }
                return arr;
            });
            return length;
        };
        _this.handleUnshift = function (value) { return function () { return _this.unshift(value); }; };
        _this.handleRemove = function (index) { return function () { return _this.remove(index); }; };
        _this.handlePop = function () { return function () { return _this.pop(); }; };
        _this.remove = _this.remove.bind(_this);
        _this.pop = _this.pop.bind(_this);
        return _this;
    }
    FieldArrayInner.prototype.remove = function (index) {
        var result;
        this.updateArrayField(function (array) {
            var copy = array ? array.slice() : [];
            if (!result) {
                result = copy[index];
            }
            if (isFunction(copy.splice)) {
                copy.splice(index, 1);
            }
            return copy;
        }, true, true);
        return result;
    };
    FieldArrayInner.prototype.pop = function () {
        var result;
        this.updateArrayField(function (array) {
            var tmp = array;
            if (!result) {
                result = tmp && tmp.pop && tmp.pop();
            }
            return tmp;
        }, true, true);
        return result;
    };
    FieldArrayInner.prototype.render = function () {
        var arrayHelpers = {
            push: this.push,
            pop: this.pop,
            swap: this.swap,
            move: this.move,
            insert: this.insert,
            replace: this.replace,
            unshift: this.unshift,
            remove: this.remove,
            handlePush: this.handlePush,
            handlePop: this.handlePop,
            handleSwap: this.handleSwap,
            handleMove: this.handleMove,
            handleInsert: this.handleInsert,
            handleReplace: this.handleReplace,
            handleUnshift: this.handleUnshift,
            handleRemove: this.handleRemove,
        };
        var _a = this.props, component = _a.component, render = _a.render, children = _a.children, name = _a.name, _b = _a.formik, _validate = _b.validate, _validationSchema = _b.validationSchema, restOfFormik = tslib_1.__rest(_b, ["validate", "validationSchema"]);
        var props = tslib_1.__assign({}, arrayHelpers, { form: restOfFormik, name: name });
        return component
            ? React.createElement(component, props)
            : render
                ? render(props)
                : children
                    ? typeof children === 'function'
                        ? children(props)
                        : !isEmptyChildren(children) ? React.Children.only(children) : null
                    : null;
    };
    FieldArrayInner.defaultProps = {
        validateOnChange: true,
    };
    return FieldArrayInner;
}(React.Component));
var FieldArray = connect(FieldArrayInner);

var ErrorMessageImpl = (function (_super) {
    tslib_1.__extends(ErrorMessageImpl, _super);
    function ErrorMessageImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ErrorMessageImpl.prototype.shouldComponentUpdate = function (props) {
        if (getIn(this.props.formik.errors, this.props.name) !==
            getIn(props.formik.errors, this.props.name) ||
            getIn(this.props.formik.touched, this.props.name) !==
                getIn(props.formik.touched, this.props.name) ||
            Object.keys(this.props).length !== Object.keys(props).length) {
            return true;
        }
        else {
            return false;
        }
    };
    ErrorMessageImpl.prototype.render = function () {
        var _a = this.props, component = _a.component, formik = _a.formik, render = _a.render, children = _a.children, name = _a.name, rest = tslib_1.__rest(_a, ["component", "formik", "render", "children", "name"]);
        var touch = getIn(formik.touched, name);
        var error = getIn(formik.errors, name);
        return !!touch && !!error
            ? render
                ? isFunction(render) ? render(error) : null
                : children
                    ? isFunction(children) ? children(error) : null
                    : component
                        ? React.createElement(component, rest, error)
                        : error
            : null;
    };
    return ErrorMessageImpl;
}(React.Component));
var ErrorMessage = connect(ErrorMessageImpl);

exports.useFormik = useFormik;
exports.Formik = Formik;
exports.yupToFormErrors = yupToFormErrors;
exports.validateYupSchema = validateYupSchema;
exports.useField = useField;
exports.Field = Field;
exports.FastField = FastField;
exports.Form = Form;
exports.withFormik = withFormik;
exports.move = move;
exports.swap = swap;
exports.insert = insert;
exports.replace = replace;
exports.FieldArray = FieldArray;
exports.isFunction = isFunction;
exports.isObject = isObject;
exports.isInteger = isInteger;
exports.isString = isString;
exports.isNaN = isNaN$1;
exports.isEmptyChildren = isEmptyChildren;
exports.isPromise = isPromise;
exports.isInputEvent = isInputEvent;
exports.getActiveElement = getActiveElement;
exports.makeCancelable = makeCancelable;
exports.getIn = getIn;
exports.setIn = setIn;
exports.setNestedObjectValues = setNestedObjectValues;
exports.connect = connect;
exports.ErrorMessage = ErrorMessage;
exports.FormikProvider = FormikProvider;
exports.FormikConsumer = FormikConsumer;
exports.useFormikContext = useFormikContext;
//# sourceMappingURL=formik.cjs.development.js.map
